#!/usr/bin/env python
"""
function:
version:1.7
author: jun ding
since: Mar.21,2012
usage: python SIOMICS.py -i <chipseq sequence> -o <output directory> 
                  -w W -m M -s S -c C -r R
input format:
<chipseq sequence> : this is chip-seq region sequences, Fasta format
<M>: choose top M ranked motifs
<W>: length of w-mer
<S>: support value for pattern mining
<C>: p-value cutoff
<R>: repeat_try times
<E>: Extension
<T>: STAMP comparsion
<Y>: Cytoscape Visulzation 
"""
import pdb,sys,os,math,getopt,shutil,random
from operator import mul
sys.path.append("code")
import StatTest
from StatTest import *
import subprocess
import threading
import datetime
from datetime import datetime

########################################################################
## check python version and make the code compatible for python 2 and 3
p_ver=sys.version_info[0]
python=sys.executable
########################################################################




########################################################################
## this code is used to get the mismathes between two input 8mers.
def mismatch(i,j):
	##function: this code is used to get the mismatch number of
	##two input mer, i and j.
	flag_list=[0 for item in range(len(i)) if i[item]!=j[item]]
	return len(flag_list)
########################################################################

########################################################################
## this code is used to build the matrices based on the clusters
## clusters format: [[freq1,freq2,...],[8mer1,8mer2,...]]
## e.g. [[8,10,...],[ACGTACGT,ACGTACGG,...]]
## mlength means the means of mer, here mlength=8

def build_matrix(clusters,mlength):
	## seq is cluster list
	## each item : frequency, 8-mer
	PWM=[]
	pseudocount=0.01
	for c in range(len(clusters)):
		cluster=clusters[c]
		seq=cluster[1] # cluster sequences, string
		freq=cluster[0] # frequencey of sequences in cluster, int
		row=[]
		for i in range(mlength):
			row_i=[0.0,0.0,0.0,0.0]
			dN={"A":0,"C":1,"G":2,"T":3}
			for j in range(len(seq)):
				Ind=dN[seq[j][i]]
				row_i[Ind]+=freq[j]
			row_i=[(round(item/sum(row_i),4)) for item in row_i]
			row_i=[item+pseudocount for item in row_i]
			row_i=[round(item/sum(row_i),4) for item in row_i]
			row.append(row_i)
		PWM.append(row)
	return PWM 


########################################################################


########################################################################

def xmer_cluster(em):
	dc={}
	for i in em:
		enum_i=[]
		t=i[1][0]
		t=[item for item in t]
		for j in range(len(t)):
			tr=t[:]
			tr[j]="A"
			enum_i.append("".join(tr))
			tr[j]="C"
			enum_i.append("".join(tr))
			tr[j]="G"
			enum_i.append("".join(tr))
			tr[j]="T"
			enum_i.append("".join(tr))
		flag=0
		j_can=[]
		for j in enum_i:
			if p_ver<=2:
				if dc.has_key(j)==True:
					j_can.append(dc[j])
					flag=1
			elif p_ver>=3:
				if j in dc:
					j_can.append(dc[j])
					flag=1
				
		#ppdb.set_trace()
		if flag==1:
			K=[]
			for jk in j_can:
				k=jk
				k=k.split("\t")[0]
				k=int(k.split(",")[0])
				K.append(k)
			jc=j_can[K.index(max(K))]
			jc=jc.split("\t")[0].split(",")[1]
			#pdb.set_trace()
			dc[jc]+="\t"+str(i[0][0])+","+i[1][0]
		if flag==0:
			dc[i[1][0]]=str(i[0][0])+","+i[1][0]
	out=[]
	for i in dc.values():
		onum=[]
		oseq=[]
		t=i.split("\t")
		for j in t:
			j=j.split(",")
			onum.append(int(j[0]))
			oseq.append(j[1])
		out.append([onum,oseq])
	for i in range(len(out)):
		out[i]=[out[i][1][0],out[i]]
		
	for i in range(len(em)):
		em[i]=em[i][1][0]
	for i in range(len(out)):
		out[i]=[em.index(out[i][0]),out[i][1]]
	out.sort()
	
	for i in range(len(out)):
		out[i]=out[i][1]
	#pdb.set_trace()
	return out
		
	


########################################################################
## this code is used to remove the redundant motifs. Some putative motifs
## found may be very similar to each other. Therefore it is necessary 
## to filter out the redundant motifs. 

def rm_red(PWM,clusters):
	## all PWMs 
	## clusters (e.g.[[8,10,...],[ACGTACGT,ACGTACGG,...]])
	## the i-th PWM correspond to i-th clusters
	
	## convert PWM to consens
	Con=[]
	cut=0.2  # cutoff used to generate the consensus
	for i in range(len(PWM)):
		Con.append(consensus(PWM[i],cut))
	
	## filter out the redundancy
	for i in range(len(Con)-1):
		for j in range(i+1,len(Con)):
			if Con[i]!=[] and Con[j]!=[]:
				if cons_compare(Con[i],Con[j]):
					Con[j]=[]
					PWM[j]=[]
					clusters[j]=[]
		if i%100==0:
			progress=float(i)/len(Con)
			progress=progress*100
			progress="%02d"%(progress)
			sys.stdout.write("\r%s%%"%(progress))
			sys.stdout.flush()
	print ("\r100%")
	PWM=[item for item in PWM if item!=[]]
	clusters=[item for item in clusters if item!=[]]
	return PWM,clusters			
		
########################################################################


########################################################################
## this module is used to convert the PWM into consensus under given 
## cutoff.
def consensus(pwm,cut):
	# pwm is the position weight matrix for a motif
	# cut is the cut used to define a occurence
	cons=""
	for i in pwm:
		ci=""
		dN="ACGT"
		for j in range(len(i)):
			if i[j]>=cut:
				ci+=dN[j]
		cons+=ci+";"
	return cons


########################################################################



#######################################################################
## this module is used to get the reverse-complement of given consenus
def rev_com(cons):
	dre={"A":"T","C":"G","G":"C","T":"A"}
	cons=cons[::-1]
	cons=cons.split(";")
	cons=[item for item in cons if item!=""]
	recon=[]
	for i in cons:
		ri=""
		for j in i:
			ri+=dre[j]
		recon.append(ri)
	recon=";".join(recon)
	recon+=";"
	return recon

########################################################################


########################################################################
## this module is used to compare to motif consensus to see whether
## there are similar.
def cons_compare(a,b):
	c=rev_com(b) # b' reverse complement
	a=a.split(";")
	b=b.split(";")
	c=c.split(";")
	a=[item for item in a if item!=""]
	b=[item for item in b if item!=""]
	c=[item for item in c if item!=""]
	mis_count=0
	mis_count_r=0
	for i in range(len(a)):
		overlap_i=[item for item in a[i] if item in b[i]]
		overlap_c_i=[item for item in a[i] if item in c[i]]
		if len(overlap_i)==0:
			mis_count+=1
		if len(overlap_c_i)==0:
			mis_count_r+=1
	if mis_count*mis_count_r>0:
		return False
	else:
		return True
			
	
########################################################################

########################################################################
## MDscan scoring schema
def mdscan(mot,mer,Bg,mlength):
	###########################################
	##PWM : Motif                             #
	##Emer: Clusters of the motif             #
	##Bg: Nuclotide Background                #
	###########################################
	w=mlength
	segments=[] # get all the motif instances
	for i in range(len(mer[0])):
		segments+=[mer[1][i]]*mer[0][i]
		
	xm=len(segments)
	
	#pdb.set_trace()
	##the first part--pi
	pi=0
	for j in range(len(mot)):
		pj=0
		for k in mot[j]:
			pijk=float(k)
			pj+=pijk*math.log(pijk)
		pi+=pj
	
	##the second part--p0i
	p0_i=0
	for j in segments:
		p0_i+=math.log(p0(j,Bg))
	
	p0_i=p0_i/xm
	
	#pdb.set_trace()
	##the final pvalue
	score=(pi-p0_i)*math.log(xm)/w
	
	return score

########################################################################


########################################################################
## p0(s), this code is subrountine for mdscan
## It is used to calculate the effect of background nuclotide frequency
def p0(s,Bg):
	p=1
	dN={"A":0,"C":1,"G":2,"T":3}
	for l in s:
		pl=Bg[dN[l]]
		p=p*pl
	return p
	
########################################################################



########################################################################
## seqPermte(seqs), this module is used to permute the input sequence

def seqPermute(seqs):
	seq_permuted=[]
	for i in range(len(seq)):
		seqi=seq[i][1]
		seqi=[item for item in seqi]
		random.shuffle(seqi)
		seqi="".join(seqi)
		seq_permuted.append([seq[i][0],seqi])
	return seq_permuted

########################################################################


########################################################################
## motifChoose(PWM,start,end) this modue is used to choose motifs 
## from PWM (from start to end)

def motifChoose(PWM,s,e):
	M=[item[:] for item in PWM[s:e]]
	return M

########################################################################

########################################################################
## update the chosen motifs in each repeat_try
##

def update_motif_chosen(PWM,motif_chosen,motifs_in_mod,start,end):
	motifs_in_mod=[int(item) for item in motifs_in_mod]
	motifs_in_mod.sort()
	M_mod=[]
	for i in range(len(motifs_in_mod)):
		M_mod.append(motif_chosen[motifs_in_mod[i]])
	start=end
	end=end+len(motif_chosen)-len(M_mod)
	M_mod+=motifChoose(PWM,start,end)
	return M_mod,start,end


########################################################################


########################################################################
## main function part
##

########################################################################
## check the format of input parameters

opt,args=getopt.getopt(sys.argv[1:],"i:o:w:m:s:c:r:e:t:y:j:")
#pdb.set_trace()
FLAG=0
java="java"

for i in opt:
	if i[0]=="-i":
		inputseqname=i[1]
		FLAG+=1
	if i[0]=="-o":
		outputdir=i[1]
		FLAG+=1
	if i[0]=="-w":
		W=int(i[1])
		FLAG+=1
	if i[0]=="-m":
		M=int(i[1])
		FLAG+=1
	if i[0]=="-s":
		S=int(i[1])
		FLAG+=1
	if i[0]=="-c":
		C=float(i[1])
		FLAG+=1
	if i[0]=="-r":
		R=int(i[1])
		FLAG+=1
	if i[0]=="-e":
		E=int(i[1])
		FLAG+=1
	if i[0]=="-t":
		T=int(i[1])
		FLAG+=1
	if i[0]=="-y":
		Y=int(i[1])
		FLAG+=1
	if i[0]=="-j":
		java=i[1]

#pdb.set_trace()
if FLAG<10:
	print("please use the following input format:")
	print(
"""
 python SIOMICS.py
 -i <input_sequence> 
 -o <output_directory>
 -w <length of the output motifs>
 -m <number_of_output_motifs>
 -s <support value>
 -c <pvalue cutoff>
 -r <iterations>
 -e <0/1> extension
 -t <0/1> STAMP comparison
 -y <0/1> cytoscape visualization
 -j <j_path> specify the java path, this is optional, if not specified, it will automatically the system environment for java path. 
""" 
	)
	sys.exit(0)


seq_name=inputseqname.split("/")[-1]


def rename_output(o):
	i=1
	while(os.path.exists(o+"_"+str(i))==True):
		i+=1
	o=o+"_"+str(i)
	return o 
	
try:	
	## read and process input chipseq sequence
	if os.path.exists(outputdir)==False:
		os.mkdir(outputdir)
	else:
		outputdir=rename_output(outputdir)
		os.mkdir(outputdir)
	##################################################################
	## output log
	log_info="Command using:\n"+"python SIOMICS.py -i %s -w %s -m %s -s %s -c %s -r %s -o %s"%(inputseqname,W,M,S,C,R,outputdir)+"\n"
	log_info+="Running started from: "+str(datetime.now())+"\n"
	
	##################################################################
	f=open(inputseqname,'r')
	lf=f.readlines()
	f.close()
	lf="".join(lf)
	lf=lf.split(">")[1:]
	lf=[item.split("\n") for item in lf]
	for i in range(len(lf)):
		lf[i]=[item.strip() for item in lf[i]]
		lf[i]=[lf[i][0],"".join(lf[i][1:]).strip()]
	seq=lf
	#pdb.set_trace()
	Bg=[0,0,0,0]
	dN={"A":0,"C":1,"G":2,"T":3}
	for i in seq:
		for j in i[1]:
			# check the version of python and make the code compatible for 
			# python 2 and python 3
			if p_ver<=2:
				if dN.has_key(j):
					Bg[dN[j]]+=1
			elif p_ver>=3:
				if j in dN:
					Bg[dN[j]]+=1
	Bg=[round(float(item)/sum(Bg),3) for item in Bg] #nuclotide freq bg
	########################################################################

	########################################################################
	## obtain all 8-mers without repeat###
	print ("Generating motif candidates....")
	Emer={} #hashtable, store frequencies of all 8-mer
	mlength=W # length of mer
	direct_hash_cut=10
	########################################################################
	## With the increasing of x, the number x-mers increase rapidly.
	## If we consider all x-mers, we will have no sufficient memory.
	## So, we only consider Top "Tmer" x-mers, those x-mers with very low frequency
	## will not be considered, since they are not likely to be the instances
	## of motifs. Here, we choose Tmer=70000. It is sufficently large based 
	## on our experiments. 
	TMer=70000 # only the Tmer x-mers  (frequency ordered) was considered
	#########################################################################

	print("phase 1:")
	if mlength<=direct_hash_cut:
		# if motif length is smaller than 10, use the original hashing
		for i in seq:
			seqi=i[1]
			for j in range(len(seqi)-mlength+1):
				mer=seqi[j:j+mlength]
				if p_ver<=2:
					if Emer.has_key(mer)==False:
						Emer[mer]=1
					else:
						Emer[mer]+=1
				elif p_ver>=3:
					if mer not in Emer:
						Emer[mer]=1
					else:
						Emer[mer]+=1
					
		########################################################################
		## sort the 8-mer based on the frequency
		Emer=sorted([[[value],[key]] for (key,value) in Emer.items() if "N" not in key], reverse=True)
		Emer=Emer[0:TMer]
		for i in range(len(Emer)-1):
			for j in range(i+1,len(Emer)):
				if (Emer[i]!="") and (Emer[j]!=""):
					if mismatch(Emer[i][1][0],Emer[j][1][0])<=1:
						Emer[i][0]+=Emer[j][0]
						Emer[i][1]+=Emer[j][1]
						Emer[j]=""
			if i%100==0:
				progress=float(i)/len(Emer)
				progress=progress*100
				progress="%02d"%(progress)
				sys.stdout.write("\r%s%%"%(progress))
				sys.stdout.flush()
		print("\r100%")
		Emer=[item for item in Emer if item!=""]

		########################################################################

	########################################################################
	else:
		# if motif length is larger than 10, we need to do some processing to avoid the memory problem.
		# The processing is based on the following strategy:\
		# Take 14-mer as an example:
		# divide 14 mer into prefix (10) and suffix (4) 
		# only consider 14-mer with large frequency prefix. 
		# for those 14-mer with very low frequency prefix, they are not possible to have large frequency. 
		##################################################################################################
		Emer_hash_cut={}
		for i in seq:
			seqi=i[1]
			for j in range(len(seqi)-direct_hash_cut+1):
				mer_hash_cut=seqi[j:j+direct_hash_cut]
				if p_ver<=2:
					if Emer_hash_cut.has_key(mer_hash_cut)==False:
						Emer_hash_cut[mer_hash_cut]=1
					else:
						Emer_hash_cut[mer_hash_cut]+=1
				elif p_ver>=3:
					if mer_hash_cut not in Emer_hash_cut:
						Emer_hash_cut[mer_hash_cut]=1
					else:
						Emer_hash_cut[mer_hash_cut]+=1
		Emer_hash_cut=sorted([[[value],[key]] for (key,value) in Emer_hash_cut.items() if "N" not in key], reverse=True)
		Emer_hash_cut=Emer_hash_cut[0:TMer]
		Emer_hash_cut_dic={}
		for i in Emer_hash_cut:
			Emer_hash_cut_dic[i[1][0]]=i[0][0]
		for i in seq:
			seqi=i[1]
			for j in range(len(seqi)-mlength+1):
				mer=seqi[j:j+mlength]
				mer_cut=mer[0:direct_hash_cut]
				if p_ver<=2:
					if Emer_hash_cut_dic.has_key(mer_cut)==True:
						if Emer.has_key(mer)==False:
							Emer[mer]=1
						else:
							Emer[mer]+=1
				elif p_ver>=3:
					if mer_cut in Emer_hash_cut_dic:
						if mer not in Emer:
							Emer[mer]=1
						else:
							Emer[mer]+=1
		Emer=sorted([[[value],[key]] for (key,value) in Emer.items() if "N" not in key], reverse=True)
		Emer=Emer[0:TMer]
		########################################################################
		## cluster the 8mers into patterns.
		## The clustering criteria is allowing one mismatch at most
		########################################################################
		Emer=xmer_cluster(Emer)
		#pdb.set_trace()
		########################################################################
			
	########################################################################


	#######################################################################
	##Generating Matrix based on the non-redundant clusters
	#pdb.set_trace()
	PWM=build_matrix(Emer,mlength)			
	#pdb.set_trace()
					
	########################################################################

	########################################################################
	##filtering out redundant motifs and corresponding clusters
	print("phase 2:")
	if mlength<=direct_hash_cut:
		PWM,Emer=rm_red(PWM,Emer)
	#pdb.set_trace()
	########################################################################

	########################################################################
	##Using MDscan to score each PWM
	print ("scoring motif candidates...")
	Score=[]
	for i in range(len(PWM)):
		score_i=mdscan(PWM[i],Emer[i],Bg,mlength)
		Score.append(score_i)
	########################################################################

	########################################################################
	## ranking the motifs
	print ("ranking motif candidates...")
	for i in range(len(Score)):
		PWM[i]=[Score[i]]+PWM[i]
	PWM.sort(reverse=True)
	########################################################################


	########################################################################
	## repeat_try start
	# repeat_try

	repeat_try=R
	update_motif=motifChoose(PWM,0,M)
	start=0
	end=M
	t=0

	########################################################################
	 ## shows # of permuted sequence used to get lambda
	# permute the input seq

	shutil.copy2("code/seq_permuted",outputdir+"/")

	########################################################################
	while(t<repeat_try):
		print ("\nround %s ..."%(t))
		########################################################################
		## choosing top M motifs
		motif_chosen=[item[:] for item in update_motif]
		print ("chooseing top %s PWMs ..."%(M))
		MMotif=[item[:] for item in motif_chosen]
		########################################################################

		########################################################################
		## predicting motif combinations.
		print ("predicting motif combinations...")

		##Generating motif file for get_lambda.py 
		if len(MMotif)==0:
			print("\nNo motifs found under current parameters, please adjust properly!")
			if os.path.exists(outputdir):
				shutil.rmtree(outputdir)
			if os.path.exists(outputdir+"_mod"):
				shutil.rmtree(outputdir+"_mod")
			os.mkdir(outputdir)
			ef=open("%s/%s.motifs"%(outputdir,seq_name),'w')
			ef.write("no motifs identified under given parameters! Please adjust properly!")
			ef.close()
			sys.exit(0)
		
		for i in range(len(MMotif)):
			MMotif[i][0]=">M"+str(i)+"\t"+str(MMotif[i][0])
			for j in range(1,len(MMotif[i])):
				MMotif[i][j]=[str(item) for item in MMotif[i][j]]
				MMotif[i][j]=" ".join(MMotif[i][j])
			MMotif[i]="\n".join(MMotif[i])
		MMotif="\n".join(MMotif)
		f=open(outputdir+"/TopMMotifs",'w')
		f.write(MMotif)
		f.close()

		## Generating motif files for crm.py
		
		## calculating lambda based on permuted sequences
		lambda_cut_motif=0.00001
		try:
			os.environ["PATH"]+=";C:\\Program Files (x86)\\Java\\jre7\\bin;"
			os.environ["PATH"]+=";C:\\Program Files\\Java\\jre7\\bin;"
			os.environ["PATH"]+=";C:\\Program Files (x86)\\Java\\jre6\\bin;"
			os.environ["PATH"]+=";C:\\Program Files\\Java\\jre6\\bin;"
			subprocess.call([java,"-version"])
			os.system(java+" -cp code getLambda "+outputdir+"/seq_permuted"+" "+outputdir+"/TopMMotifs"+" "+str(lambda_cut_motif)+" "+outputdir)
		except:
			os.system("code\\get_lambda.py "+outputdir+"/seq_permuted"+" "+outputdir+"/TopMMotifs"+" "+str(lambda_cut_motif)+" "+outputdir)

		## preparing and precessing PWMs and lambda
		f=open(outputdir+"/TopMMotifs",'r')
		lf=f.readlines()
		f.close()
		lf="".join(lf)
		lf=lf.split(">")[1:]
		matrix=lf


		f=open(outputdir+"/TopMMotifs_lambda",'r')
		lf=f.readlines()
		f.close()
		lf=[item.strip() for item in lf]
		lf=[item.split("\t") for item in lf]
		lam=lf

		for i in range(len(matrix)):
			matrix[i]=matrix[i].split("\n")
			tmp=matrix[i][0].split("\t")
			matrix[i][0]=">M"+str(i)+"\t"+lam[i][0]+"\t"+lam[i][1]+"\t"+tmp[1]
			matrix[i]="\n".join(matrix[i])
			#pbd.set_trace()
		matrix="\n".join(matrix)
		f=open(outputdir+"/TopMMotifs",'w')
		f.write(matrix)
		f.close()

		for i in range(len(lam)):
			lam[i]=">M"+str(i)+"\t"+lam[i][1]+"\t"+"NA"
		lam="\n".join(lam)
		f=open(outputdir+"/TopMMotifs_lambda",'w')
		f.write(lam)
		f.close()

		## run crm.py to get the motif combinations
		os.system("code\\crm.py -i "+'"%s"'%(inputseqname)+" -o "+outputdir+"_mod"+" -s "+str(S)+" -c "+str(C)+" -m "+outputdir+"/TopMMotifs"+" -l "+outputdir+"/TopMMotifs_lambda"+" -f txt"+" -d N")
		########################################################################


		########################################################################]
		## find out M1 out ot M motifs occured in the combinations
		print ("updating motif candidates...")
		f=open(outputdir+"_mod/mod",'r')
		lf=f.readlines()
		f.close()
		mod=[item.strip() for item in lf]
		mod=[item for item in mod if item!="()"]
		mod=[item.split(" ")[:-1] for item in mod]
		motifs_in_mod=[]
		for i in mod:
			for j in i:
				if j not in motifs_in_mod:
					motifs_in_mod.append(j)		
		if len(motifs_in_mod)==M:
			break
		update_motif,start,end=update_motif_chosen(PWM,motif_chosen,motifs_in_mod,start,end)
		t+=1
		if end>=len(PWM):
			break
	##########################################################################################
	##output motifs_in_mod motifs

	motifs_in_mod=[int(item) for item in motifs_in_mod]
	motifs_in_mod.sort()
	MIM=[motif_chosen[item] for item in motifs_in_mod]
	motif_chosen=MIM
	## get final motif prediction and motif prediction
	MMotif=[item[:] for item in motif_chosen]
	########################################################################

	########################################################################
	## predicting motif combinations.
	print ("predicting motif combinations...")

	if len(MMotif)==0:
			"""
			print("\nNo motifs found under current parameters, please adjust properly!")
			if os.path.exists(outputdir):
				shutil.rmtree(outputdir)
			if os.path.exists(outputdir+"_mod"):
				shutil.rmtree(outputdir+"_mod")
			os.mkdir(outputdir)
			ef=open("%s/%s.motifs"%(outputdir,seq_name),'w')
			ef.write("no motifs identified under given parameters! Please adjust properly!")
			ef.close()
			"""
			sys.exit(0)
	##Generating motif file for get_lambda.py 

	for i in range(len(MMotif)):
		MMotif[i][0]=">M"+str(i)+"\t"+str(MMotif[i][0])
		for j in range(1,len(MMotif[i])):
			MMotif[i][j]=[str(item) for item in MMotif[i][j]]
			MMotif[i][j]=" ".join(MMotif[i][j])
		MMotif[i]="\n".join(MMotif[i])
	MMotif="\n".join(MMotif)
	f=open(outputdir+"/TopMMotifs",'w')
	f.write(MMotif)
	f.close()
	## Generating motif files for crm.py

	## calculating lambda based on permuted sequences
	Tmc=2 # lambda cutoff for module finding is looser than that of motif finding so that we will not miss predict too many combinations because of too stringent lambda cutoff.
	lambda_cut_mc=Tmc*lambda_cut_motif
	try:
		subprocess.call([java,"-version"])
		os.system(java+" -cp code getLambda "+outputdir+"/seq_permuted"+" "+outputdir+"/TopMMotifs"+" "+str(lambda_cut_mc)+" "+outputdir)
	except:
		os.system("code\\get_lambda.py "+outputdir+"/seq_permuted"+" "+outputdir+"/TopMMotifs"+" "+str(lambda_cut_mc)+" "+outputdir)

	## preparing and precessing PWMs and lambda
	f=open(outputdir+"/TopMMotifs",'r')
	lf=f.readlines()
	f.close()
	lf="".join(lf)
	lf=lf.split(">")[1:]
	matrix=lf


	f=open(outputdir+"/TopMMotifs_lambda",'r')
	lf=f.readlines()
	f.close()
	lf=[item.strip() for item in lf]
	lf=[item.split("\t") for item in lf]
	lam=lf

	for i in range(len(matrix)):
		matrix[i]=matrix[i].split("\n")
		tmp=matrix[i][0].split("\t")
		matrix[i][0]=">M"+str(i)+"\t"+lam[i][0]+"\t"+lam[i][1]+"\t"+tmp[1]
		matrix[i]="\n".join(matrix[i])
		#pbd.set_trace()
	matrix="\n".join(matrix)
	f=open(outputdir+"/TopMMotifs",'w')
	f.write(matrix)
	f.close()


	for i in range(len(lam)):
		lam[i]=">M"+str(i)+"\t"+lam[i][1]+"\t"+"NA"
	lam="\n".join(lam)
	f=open(outputdir+"/TopMMotifs_lambda",'w')
	f.write(lam)
	f.close()
	#pdb.set_trace()
	## run crm.py to get the motif combinations
	os.system("code\\crm.py -i "+'"%s"'%(inputseqname)+" -o "+outputdir+"_mod"+" -s "+str(S)+" -c "+str(C)+" -m "+outputdir+"/TopMMotifs"+" -l "+outputdir+"/TopMMotifs_lambda"+" -f txt"+" -d N")

	
	#=====================================================================
	
	
	########################################################################
	## output predicted motifs and motif combinations
	
	FN=inputseqname.split("/")[-1]
	#pdb.set_trace()
	shutil.move(outputdir+"_mod/mod",outputdir+"/mod")
	os.system('code\\get_tfbs.py %s_mod\\%s_MotifHits_4_non_redundant %s_mod\\%s_MotifHitsPositions_4_non_redundant "%s" %s %s'%(outputdir,FN,outputdir,FN,inputseqname,str(W),outputdir))
	shutil.move(outputdir+"_mod/"+seq_name+"_MotifHits_4_non_redundant", outputdir+"/"+seq_name+"_MotifHits_4_non_redundant")
	shutil.move(outputdir+"_mod/"+seq_name+"_MotifHitsPositions_4_non_redundant", outputdir+"/"+seq_name+"_MotifHitsPositions_4_non_redundant")
	shutil.rmtree(outputdir+"_mod")
	os.remove(outputdir+"/seq_permuted")
	os.remove(outputdir+"/TopMMotifs_lambda")


	## processing the output files
	f=open(outputdir+"/mod",'r')
	lf=f.readlines()
	f.close()
	lf=[item.strip() for item in lf]
	lf=[item for item in lf if item!="()"]
	lf=[item.split("\t") for item in lf]
	for i in range(len(lf)):
		lf[i][0]=lf[i][0].split(" ")
		for j in range(len(lf[i][0][:-1])):
			lf[i][0][j]="M"+lf[i][0][j]
		lf[i][0]=" ".join(lf[i][0])
		lf[i]="\t".join(lf[i])
	lf="\n".join(lf)
	#inputseqname=inputseqname.split("/")[-1]
	f=open(outputdir+"/"+FN+".mc",'w')
	f.write(lf)
	f.close()
	
	#-------------------------------------------------------------
	print ("adjusting predicted motifs and motif modules...")
	os.rename(outputdir+"/TopMMotifs",outputdir+"/"+seq_name+".motifs")
	#pdb.set_trace()
	
	if E==1:
		os.system("code\\batch_extend_shorten.py %s/%s.mc %s/%s.motifs %s/%s_MotifHits_4_non_redundant %s/%s_MotifHitsPositions_4_non_redundant %s %s" %(outputdir,seq_name,outputdir,seq_name,outputdir,seq_name,outputdir,seq_name,inputseqname,W))
		os.mkdir(outputdir+"/Result_after_length_adjust/")
		shutil.move(outputdir+"/"+seq_name+".motifs.extended.nonred.shorten.final", outputdir+"/Result_after_length_adjust/"+seq_name+".motifs.adjusted")
		shutil.move(outputdir+"/"+seq_name+".mc.final", outputdir+"/Result_after_length_adjust/"+seq_name+".mc.adjusted")
		os.remove(outputdir+"/"+seq_name+".motifs.extended.nonred")
		os.remove(outputdir+"/"+seq_name+".motifs.extended.nonred.shorten")
		os.remove(outputdir+"/"+seq_name+"_MotifHits_4_non_redundant")
		os.remove(outputdir+"/"+seq_name+"_MotifHitsPositions_4_non_redundant")
	
	os.remove(outputdir+"/mod")	
	print ("Prediction is done!")
	log_info+="Running finished at: "+str(datetime.now())
	f=open(outputdir+"/running.log",'w')
	f.write(log_info)
	f.close()
	
	print ("work is done!")
except:
	"""
	print("\nNo motifs found under current parameters, please adjust properly!")
	if os.path.exists(outputdir):
		shutil.rmtree(outputdir)
	if os.path.exists(outputdir+"_mod"):
		shutil.rmtree(outputdir+"_mod")
	os.mkdir(outputdir)
	ef=open("%s/%s.motifs"%(outputdir,seq_name),'w')
	ef.write("no motifs identified under given parameters! Please adjust properly!")
	ef.close()
	"""
	sys.exit(0)
#########################################################################################
## compare to  known databases and visualization





if os.path.exists(outputdir+"/"+FN+".motifs") and os.path.exists(outputdir+"/"+FN+".mc"):
	if T==1:
		subprocess.call([python,"code\\motif_visualize_compare.py",outputdir+"/"+FN+".motifs"])
		subprocess.call([python,"code\\cytoscape_script.py",outputdir+"/"+FN+".mc"])
print ("\n All work is done! \n\n")
	
